import os
from setuptools import setup, find_packages
from outliers import (
    __version__,
    __license__,
    __author__,
    __author_email__,
)

BASE_PATH = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(BASE_PATH, 'README.rst')).read()
CHANGES = open(os.path.join(BASE_PATH, 'CHANGES.rst')).read()

__classifiers__ = [
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
]

setup(
    name='outlier_utils',
    version=__version__,
    author=__author__,
    author_email=__author_email__,
    url='https://github.com/c-bata/outlier-utils',
    description='Remove outliers with some methods',
    long_description=README + '\n\n' + CHANGES,
    packages=find_packages(exclude=['test*']),
    install_requirements=['numpy', 'scipy', 'pandas'],
    keywords='outlier grubbs pandas numpy',
    license=__license__,
    include_package_data=True,
    test_suite='tests',
)

